'use strict';

var _ansiEscapes = require('ansi-escapes');

var _configstore = require('configstore');

var _configstore2 = _interopRequireDefault(_configstore);

var _chalk = require('chalk');

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _defaultPager = require('default-pager');

var _defaultPager2 = _interopRequireDefault(_defaultPager);

var _stream = require('stream');

var _tempmail = require('tempmail.js');

var _tempmail2 = _interopRequireDefault(_tempmail);

var _toClipboard = require('to-clipboard');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

process.env.PAGER = process.env.PAGER || 'less';
process.env.LESS = process.env.LESS || 'FRX';

var cli = (0, _meow2.default)('\n    Usage\n      tempmail\n\n    Options\n      --create, -c      Generate a new email address\n      --get-mail, -g    Get messages\n      --delete-all, -D  Delete all messages\n\n    Examples\n      tempmail\n      tempmail --get-mail\n', {
  alias: {
    c: 'create',
    g: 'get-mail',
    D: 'delete-all'
  }
});

var options = new _configstore2.default(cli.pkg.name);
var account = new _tempmail2.default(!cli.flags.create && options.get('email'));
options.set('email', account.address);

if (cli.flags.getMail) {
  exitByQ();
  getMessages(account, listMessages);
} else if (cli.flags.deleteAll) {
  getMessages(account, deleteMessages);
} else {
  printAddress(account.address);
}

function getMessages(account, fn) {
  account.getMail().then(function (messages) {
    if (messages.error) {
      console.log(messages.error);
    } else {
      fn.apply(undefined, _toConsumableArray(messages));
    }
  });
}

function listMessages() {
  for (var _len = arguments.length, messages = Array(_len), _key = 0; _key < _len; _key++) {
    messages[_key] = arguments[_key];
  }

  _inquirer2.default.prompt([{
    type: 'list',
    name: 'message',
    message: 'Your messages:',
    choices: generateChoices(messages)
  }], function (answer) {
    printMessage(answer.message, messages);
  });
}

function generateChoices(messages) {
  return messages.sort(function (a, b) {
    return b.mail_timestamp - a.mail_timestamp;
  }).map(function (message) {
    return {
      name: message.mail_from + ' ' + (0, _chalk.dim)(message.mail_subject),
      value: message
    };
  });
}

function printMessage(message, messages) {
  var stream = new _stream.Readable({ encoding: 'utf8' });

  stream.push(message.mail_from + '\n' + (0, _chalk.dim)(message.mail_subject) + '\n\n' + message.mail_text + '\n  ');

  stream.push(null);
  stream.pipe((0, _defaultPager2.default)(function () {
    cleanupOutput();
    listMessages.apply(undefined, _toConsumableArray(messages));
  }));
}

function deleteMessages(message) {
  for (var _len2 = arguments.length, messages = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    messages[_key2 - 1] = arguments[_key2];
  }

  if (!message) {
    console.log('All messages have been deleted');
  } else {
    account.deleteMessage(message.mail_id).then(function () {
      return setTimeout(function () {
        return deleteMessages.apply(undefined, messages);
      }, 1000);
    });
  }
}

function printAddress(address) {
  try {
    (0, _toClipboard.sync)(address);
    console.log(address + ' ' + (0, _chalk.dim)('(copied to clipboard)'));
  } catch (err) {
    console.log(address);
  }
}

function exitByQ() {
  process.stdin.on('keypress', function (ch) {
    var key = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    if (key.name === 'q') process.exit();
  });
}

function cleanupOutput() {
  process.stdout.write(_ansiEscapes.cursorPrevLine + _ansiEscapes.eraseLine);
}
